import spacy
import neologdn
import re
import emoji
import mojimoji

class JapaneseCorpus:

    def __init__(self):
        self.nlp = spacy.load('ja_ginza')


    def preprocessing(self, text):
        text = re.sub(r'\n', "", text)
        text = re.sub(r'\r', "", text)
        text = re.sub(r'\s', "", text)
        text = text.lower()
        text = mojimoji.zen_to_han(text, kana=True)
        text = mojimoji.han_to_zen(text, digit=False, ascii=False)
        text = neologdn.normalize(text)
        text = re.sub(r"(https?|ftp)(:\/\/[-_\.!~*\'()a-zA-Z0-9;\/?:\@&=\+$,%#]+)", "" , text)
        text = re.sub(r"#(\w+)", "" , text)
        text = re.sub(r"@([A-Za-z0-9_]+) ", "" , text)
        text = ''.join(c for c in text if c not in emoji.UNICODE_EMOJI)
        text = text.strip()

        return text


    def make_sentence_list(self, sentences):
        doc = self.nlp(sentences)
        self.ginza_sents_object = doc.sents
        sentence_list = [s for s in doc.sents]

        return sentence_list


    def make_corpus(self):
        corpus = []
        for s in self.ginza_sents_object:
            # 対象は名詞、副詞、形容詞、動詞のみ。ブレをなくすため基本形を抽出する。
            tokens = [t.lemma_ for t in s if t.pos_ in ('NOUN', 'ADV', 'ADJ', 'VERB')]
            corpus.append(' '.join(tokens) + '。')

        return corpus



class EnglishCorpus(JapaneseCorpus):

    def __init__(self):
        self.nlp = spacy.load('en_core_web_sm')


    def preprocessing(self, text):
        text = re.sub(r'\n', "", text)
        text = re.sub(r'\r', "", text)
        text = text.lower()
        text = mojimoji.han_to_zen(text, digit=False, ascii=False)
        text = mojimoji.zen_to_han(text, kana=True)
        text = neologdn.normalize(text)
        text = re.sub(r"(https?|ftp)(:\/\/[-_\.!~*\'()a-zA-Z0-9;\/?:\@&=\+$,%#]+)", "" , text)
        text = re.sub(r"#(\w+)", "" , text)
        text = re.sub(r"@([A-Za-z0-9_]+) ", "" , text)
        text = ''.join(c for c in text if c not in emoji.UNICODE_EMOJI)
        text = text.strip()

        return text


    def make_corpus(self):
        corpus = []
        for s in self.ginza_sents_object:
            # 対象は名詞、副詞、形容詞、動詞のみ。ブレをなくすため基本形を抽出する。
            tokens = [str(t) for t in s]
            corpus.append(' '.join(tokens))

        return corpus



from sumy.parsers.plaintext import PlaintextParser
from sumy.nlp.tokenizers import Tokenizer
from sumy.utils import get_stop_words

# algorithms
from sumy.summarizers.lex_rank import LexRankSummarizer
from sumy.summarizers.text_rank import TextRankSummarizer
from sumy.summarizers.lsa import LsaSummarizer
from sumy.summarizers.kl import KLSummarizer
from sumy.summarizers.luhn import LuhnSummarizer
from sumy.summarizers.reduction import ReductionSummarizer
from sumy.summarizers.sum_basic import SumBasicSummarizer

from easySum.corpus import JapaneseCorpus, EnglishCorpus


algorithm_dic = {"lex": LexRankSummarizer(), "tex": TextRankSummarizer(), "lsa": LsaSummarizer(),\
                 "kl": KLSummarizer(), "luhn": LuhnSummarizer(), "redu": ReductionSummarizer(),\
                 "sum": SumBasicSummarizer()}


def summarize_sentences(sentences, sentences_count=3, algorithm="lex", language="japanese"):
    if language == "japanese":
        corpus_maker = JapaneseCorpus()
    else:
        corpus_maker = EnglishCorpus()

    preprocessed_sentences = corpus_maker.preprocessing(sentences)
    preprocessed_sentence_list = corpus_maker.make_sentence_list(preprocessed_sentences)
    corpus = corpus_maker.make_corpus()

    parser = PlaintextParser.from_string(' '.join(corpus), Tokenizer(language))

    # アルゴリズム選択
    try:
        summarizer = algorithm_dic[algorithm]
    except KeyError:
        print("algorithm name:'{}'is not found.".format(algorithm))

    summarizer.stop_words = get_stop_words(language)

    summary = summarizer(document=parser.document, sentences_count=sentences_count)

    if language == "japanese":
        return "".join([str(preprocessed_sentence_list[corpus.index(sentence.__str__())]) for sentence in summary])
    else:
        return " ".join([sentence.__str__() for sentence in summary])



from setuptools import setup
from codecs import open
from os import path

package_name = "easySum"

root_dir = path.abspath(path.dirname(__file__))

def _requirements():
    return [name.rstrip() for name in open(path.join(root_dir, 'requirements.txt')).readlines()]


with open('README.md', encoding='utf-8') as f:
    long_description = f.read()


setup(
    name=package_name,
    version='0.1.8',
    description='You can easily summarize the text.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/hcpmiyuki/easySum',
    author='miyuki kimura',
    author_email='miyuki.kimura@deepbluets.page',
    license='MIT',
    keywords='nlp python summarizer',
    packages=[package_name],
    install_requires=_requirements(),
    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
